# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2012-2024

from __future__ import annotations

from typing import Any

whole_html_template = """<!doctype html>
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">
{custom_headers}<title>{title}</title>
{stylesheet}{javascript}</head>
<body>
{body}
</body>
</html>
"""


class AbstractMarkup:
    """Abstract class for markup languages.

    :param filename: optional name of the file
    """

    #: name of the markup visible to user
    name: str
    #: various attributes, like links to website and syntax documentation
    attributes: dict[int, Any]
    #: indicates which file extensions are associated with the markup
    file_extensions: tuple[str, ...]
    #: the default file extension
    default_extension: str

    def __init__(self, filename: str | None = None):
        self.filename = filename

    @staticmethod
    def available() -> bool:
        """
        :returns: whether the markup is ready for use

                  (for example, whether the required third-party
                  modules are importable)
        """
        return True

    def convert(self, text: str) -> ConvertedMarkup:
        """
        :returns: a ConvertedMarkup instance (or a subclass thereof)
                  containing the markup converted to HTML
        """
        raise NotImplementedError


class ConvertedMarkup:
    """This class encapsulates the title, body, stylesheet and javascript
    of a converted document.

    Instances of this class are created by :meth:`.AbstractMarkup.convert`
    method, usually it should not be instantiated directly.
    """

    def __init__(
        self,
        body: str,
        title: str = "",
        stylesheet: str = "",
        javascript: str = "",
    ):
        self.title = title
        self.stylesheet = stylesheet
        self.javascript = javascript
        self.body = body

    def get_document_title(self) -> str:
        """
        :returns: the document title
        """
        return self.title

    def get_document_body(self) -> str:
        """
        :returns: the contents of the ``<body>`` HTML tag
        """
        return self.body

    def get_stylesheet(self) -> str:
        """
        :returns: the contents of ``<style type="text/css">`` HTML tag
        """
        return self.stylesheet

    def get_javascript(self, webenv: bool = False) -> str:
        """
        :returns: one or more HTML tags to be inserted into the document
                  ``<head>``.
        :param webenv: if true, the specific markups may optimize the
                       document for being used in the World Wide Web (for
                       example, a remote version of MathJax script can be
                       inserted instead of the local one).
        """
        return self.javascript

    def get_whole_html(
        self,
        custom_headers: str = "",
        include_stylesheet: bool = True,
        fallback_title: str = "",
        webenv: bool = False,
    ) -> str:
        """
        :returns: the full contents of the HTML document (unless overridden
                  this is a combination of the previous methods)
        :param custom_headers: custom HTML to be inserted into the document
                               ``<head>``
        :param include_stylesheet: if false, the stylesheet will not
                                   be included in the document ``<head>``
        :param fallback_title: when impossible to get the ``<title>`` from
                               the document, this string can be used as a
                               fallback
        :param webenv: like in :meth:`~.ConvertedMarkup.get_javascript`
                       above
        """
        stylesheet = (
            '<style type="text/css">\n' + self.get_stylesheet() + "</style>\n"
            if include_stylesheet
            else ""
        )

        context = {
            "body": self.get_document_body(),
            "title": self.get_document_title() or fallback_title,
            "javascript": self.get_javascript(webenv),
            "stylesheet": stylesheet,
            "custom_headers": custom_headers,
        }
        return whole_html_template.format(**context)



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dave Kuhlman, 2022

import importlib
import warnings
from io import StringIO

import markups.common as common
from markups.abstract import AbstractMarkup, ConvertedMarkup


class AsciiDocMarkup(AbstractMarkup):
    """Markup class for AsciiDoc language.
    Inherits :class:`~markups.abstract.AbstractMarkup`.
    """

    name = "asciidoc"
    attributes = {
        common.LANGUAGE_HOME_PAGE: "https://asciidoc.org",
        common.MODULE_HOME_PAGE: "https://asciidoc-py.github.io",
        common.SYNTAX_DOCUMENTATION: "https://asciidoc-py.github.io/userguide.html",
    }

    file_extensions = (".adoc", ".asciidoc")
    default_extension = ".adoc"

    @staticmethod
    def available() -> bool:
        try:
            importlib.import_module("asciidoc")
            importlib.import_module("lxml")
        except ImportError:
            return False
        return True

    def convert(self, text: str) -> ConvertedMarkup:
        import asciidoc
        from lxml import etree

        outfile = StringIO()
        infile = StringIO(text)
        opts = [
            ("--backend", "html5"),
            ("--attribute", r"newline=\n"),
            ("--attribute", "footer-style=none"),
            ("--out-file", outfile),
        ]
        try:
            asciidoc.execute(None, opts, [infile])
        except SystemExit as ex:
            warnings.warn(str(ex.__context__), SyntaxWarning)
            pass
        result = outfile.getvalue()
        parser = etree.HTMLParser()
        root_element = etree.fromstring(result, parser)
        head_element = root_element.xpath("./head")[0]
        title_element = root_element.xpath("./head/title")[0]
        style_elements = root_element.xpath("./head/style")
        javascript_elements = root_element.xpath("./head/script")
        body_element = root_element.xpath("./body")[0]
        head = ""
        for child in head_element.getchildren():
            head += etree.tostring(
                child,
                encoding="unicode",
                method="html",
            )
        body = ""
        for child in body_element.getchildren():
            body += etree.tostring(
                child,
                encoding="unicode",
                method="html",
            )
        title = title_element.text
        stylesheet = ""
        for style_element in style_elements:
            stylesheet += style_element.text
        javascript = ""
        for javascript_element in javascript_elements:
            javascript += etree.tostring(
                javascript_element,
                encoding="unicode",
                method="html",
            )
        return ConvertedMarkup(body, title, stylesheet, javascript)



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2012-2024

from __future__ import annotations

from typing import Any

import markups.common as common
from markups.abstract import AbstractMarkup, ConvertedMarkup

try:
    from docutils.core import publish_parts
    from docutils.writers.html5_polyglot import HTMLTranslator, Writer

    HAVE_DOCUTILS = True
except ImportError:
    HAVE_DOCUTILS = False

if HAVE_DOCUTILS:

    class CustomHTMLTranslator(HTMLTranslator):  # type: ignore
        def starttag(  # type: ignore
            self,
            node,
            tagname,
            suffix="\n",
            empty=False,
            **attributes,
        ):
            if getattr(node, "line", None) is not None:
                attributes["data-posmap"] = node.line
            return super().starttag(node, tagname, suffix, empty, **attributes)


class ReStructuredTextMarkup(AbstractMarkup):
    """Markup class for reStructuredText language.
    Inherits :class:`~markups.abstract.AbstractMarkup`.

    :param settings_overrides: optional dictionary of overrides for the
                               `Docutils settings`_
    :type settings_overrides: dict

    .. _`Docutils settings`: https://docutils.sourceforge.io/docs/user/config.html
    """

    name = "reStructuredText"
    attributes = {
        common.LANGUAGE_HOME_PAGE: "https://docutils.sourceforge.io/rst.html",
        common.MODULE_HOME_PAGE: "https://docutils.sourceforge.io/",
        common.SYNTAX_DOCUMENTATION: "https://docutils.sourceforge.io/docs/ref/rst/restructuredtext.html",
    }

    file_extensions = (".rst", ".rest")
    default_extension = ".rst"

    @staticmethod
    def available() -> bool:
        return HAVE_DOCUTILS

    def __init__(
        self,
        filename: str | None = None,
        settings_overrides: dict[str, Any] | None = None,
    ):
        self.overrides = settings_overrides or {}
        self.overrides.update(
            {
                "math_output": "MathJax " + common.MATHJAX_WEB_URL,
                "syntax_highlight": "short",
                "halt_level": 5,  # Never convert system messages to exceptions
                "stylesheet_path": "minimal.css",  # Do not include plain.css
            },
        )
        AbstractMarkup.__init__(self, filename)
        self.writer = Writer()
        self.writer.translator_class = CustomHTMLTranslator

    def convert(self, text: str) -> ConvertedReStructuredText:
        parts = publish_parts(
            text,
            source_path=self.filename,
            writer=self.writer,
            settings_overrides=self.overrides,
        )

        # Determine head
        head = parts["head"]

        # Determine body
        body = parts["html_body"]

        # Determine title
        title = parts["title"]

        # Determine stylesheet
        origstyle = parts["stylesheet"]
        # Cut off <style> and </style> tags
        stylestart = '<style type="text/css">'
        stylesheet = ""
        if stylestart in origstyle:
            stylesheet = origstyle[
                origstyle.find(stylestart) + 25 : origstyle.rfind("</style>")
            ]
        stylesheet += common.get_pygments_stylesheet(".code")

        return ConvertedReStructuredText(head, body, title, stylesheet)


class ConvertedReStructuredText(ConvertedMarkup):
    def __init__(self, head: str, body: str, title: str, stylesheet: str):
        ConvertedMarkup.__init__(self, body, title, stylesheet)
        self.head = head

    def get_javascript(self, webenv: bool = False) -> str:
        if common.MATHJAX_WEB_URL not in self.head:
            return ""
        mathjax_url, mathjax_version = common.get_mathjax_url_and_version(webenv)
        if mathjax_version == 2:
            mathjax_url += "?config=TeX-AMS_CHTML"
        async_attr = " async" if mathjax_version == 3 else ""
        script_tag = '<script type="text/javascript" src="%s"%s></script>\n'
        return script_tag % (mathjax_url, async_attr)



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2012-2025

import os.path

# Some common constants and functions
(LANGUAGE_HOME_PAGE, MODULE_HOME_PAGE, SYNTAX_DOCUMENTATION) = range(3)
CONFIGURATION_DIR = (
    os.getenv("XDG_CONFIG_HOME")
    or os.getenv("APPDATA")
    or os.path.expanduser("~/.config")
)
MATHJAX2_LOCAL_FILES = (
    "/usr/share/javascript/mathjax/MathJax.js",  # Debian libjs-mathjax
    "/usr/share/mathjax2/MathJax.js",  # Arch Linux mathjax2
)
MATHJAX3_LOCAL_FILES = (
    "/usr/share/nodejs/mathjax-full/es5/tex-chtml.js",  # Debian node-mathjax-full
    "/usr/share/mathjax/tex-chtml.js",  # Arch Linux mathjax
)
MATHJAX_WEB_URL = "https://cdn.jsdelivr.net/npm/mathjax@3/es5/tex-chtml.js"

PYGMENTS_STYLE = "default"


def get_pygments_stylesheet(selector: str | None, style: str | None = None) -> str:
    if style is None:
        style = PYGMENTS_STYLE
    if style == "":
        return ""
    try:
        from pygments.formatters import HtmlFormatter
    except ImportError:
        return ""
    else:
        defs = HtmlFormatter(style=style).get_style_defs(selector)
        assert isinstance(defs, str)
        return defs + "\n"


def get_mathjax_url_and_version(webenv: bool) -> tuple[str, int]:
    if not webenv:
        for path in MATHJAX3_LOCAL_FILES:
            if os.path.exists(path):
                return f"file://{path}", 3
        for path in MATHJAX2_LOCAL_FILES:
            if os.path.exists(path):
                return f"file://{path}", 2
    return MATHJAX_WEB_URL, 3



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2012-2024

from __future__ import annotations

import importlib
import os
import re
import warnings
from collections.abc import Iterable, Iterator
from typing import Any

import markups.common as common
from markups.abstract import AbstractMarkup, ConvertedMarkup

try:
    import yaml

    HAVE_YAML = True
except ImportError:
    HAVE_YAML = False

MATHJAX2_CONFIG = """<script type="text/x-mathjax-config">
MathJax.Hub.Config({
  config: ["MMLorHTML.js"],
  jax: ["input/TeX", "input/AsciiMath", "output/HTML-CSS", "output/NativeMML"],
  extensions: ["MathMenu.js", "MathZoom.js"],
  TeX: {
    extensions: ["AMSmath.js", "AMSsymbols.js"],
    equationNumbers: {autoNumber: "AMS"}
  }
});
</script>
"""

# Taken from:
# https://docs.mathjax.org/en/latest/upgrading/v2.html?highlight=upgrading#changes-in-the-mathjax-api
MATHJAX3_CONFIG = """
<script>
MathJax = {
  options: {
    renderActions: {
      find: [10, function (doc) {
        for (const node of document.querySelectorAll('script[type^="math/tex"]')) {
          const display = !!node.type.match(/; *mode=display/);
          const math = new doc.options.MathItem(node.textContent, doc.inputJax[0], display);
          const text = document.createTextNode('');
          node.parentNode.replaceChild(text, node);
          math.start = {node: text, delim: '', n: 0};
          math.end = {node: text, delim: '', n: 0};
          doc.math.push(math);
        }
      }, '']
    }
  }
};
</script>
"""  # noqa: E501

extensions_re = re.compile(r"required.extensions: (.+)", flags=re.IGNORECASE)
extension_name_re = re.compile(r"[a-z0-9_.]+(?:\([^)]+\))?", flags=re.IGNORECASE)

_canonicalized_ext_names: dict[str, str] = {}

_name_and_config = tuple[str, dict[str, Any]]


class MarkdownMarkup(AbstractMarkup):
    """Markup class for Markdown language.
    Inherits :class:`~markups.abstract.AbstractMarkup`.

    :param extensions: list of extension names
    :type extensions: list
    """

    name = "Markdown"
    attributes = {
        common.LANGUAGE_HOME_PAGE: "https://daringfireball.net/projects/markdown/",
        common.MODULE_HOME_PAGE: "https://github.com/Python-Markdown/markdown",
        common.SYNTAX_DOCUMENTATION: "https://daringfireball.net/projects/markdown/syntax",
    }

    file_extensions = (".md", ".mkd", ".mkdn", ".mdwn", ".mdown", ".markdown")
    default_extension = ".mkd"

    @staticmethod
    def available() -> bool:
        try:
            import markdown

            importlib.import_module("mdx_math")
        except ImportError:
            return False
        return getattr(markdown, "__version_info__", (2,)) >= (3,)

    def _load_extensions_list_from_txt_file(
        self,
        filename: str,
    ) -> Iterator[_name_and_config]:
        with open(filename) as extensions_file:
            for line in extensions_file:
                if not line.startswith("#"):
                    yield self._split_extension_config(line.rstrip())

    def _load_extensions_list_from_yaml_file(
        self,
        filename: str,
    ) -> Iterator[_name_and_config]:
        with open(filename) as extensions_file:
            try:
                data = yaml.safe_load(extensions_file)
            except yaml.YAMLError as ex:
                warnings.warn(f"Failed parsing {filename}: {ex}", SyntaxWarning)
                raise OSError from ex
        if isinstance(data, list):
            for item in data:
                if isinstance(item, dict):
                    yield from item.items()
                elif isinstance(item, str):
                    yield item, {}

    def _get_global_extensions(
        self,
        filename: str | None,
    ) -> Iterator[_name_and_config]:
        local_directory = os.path.dirname(filename) if filename else ""
        choices = [
            os.path.join(local_directory, "markdown-extensions.yaml"),
            os.path.join(local_directory, "markdown-extensions.txt"),
            os.path.join(common.CONFIGURATION_DIR, "markdown-extensions.yaml"),
            os.path.join(common.CONFIGURATION_DIR, "markdown-extensions.txt"),
        ]
        for choice in choices:
            if choice.endswith(".yaml") and not HAVE_YAML:
                continue
            try:
                if choice.endswith(".txt"):
                    yield from self._load_extensions_list_from_txt_file(choice)
                else:
                    yield from self._load_extensions_list_from_yaml_file(choice)
            except OSError:
                continue  # Cannot open file, move to the next choice
            else:
                break  # File loaded successfully, skip the remaining choices

    def _get_document_extensions(self, text: str) -> Iterator[_name_and_config]:
        lines = text.splitlines()
        match = extensions_re.search(lines[0]) if lines else None
        if match:
            extensions = extension_name_re.findall(match.group(1))
            yield from self._split_extensions_configs(extensions)

    def _canonicalize_extension_name(self, extension_name: str) -> str | None:
        prefixes = ("markdown.extensions.", "", "mdx_")
        for prefix in prefixes:
            try:
                module = importlib.import_module(prefix + extension_name)
                if not hasattr(module, "makeExtension"):
                    continue
            except (ImportError, ValueError, TypeError):
                pass
            else:
                return prefix + extension_name
        return None

    def _split_extension_config(self, extension_name: str) -> _name_and_config:
        """Splits the configuration options from the extension name."""
        lb = extension_name.find("(")
        if lb == -1:
            return extension_name, {}
        extension_name, parameters = extension_name[:lb], extension_name[lb + 1 : -1]
        pairs = [x.split("=") for x in parameters.split(",")]
        return extension_name, {x.strip(): y.strip() for (x, y) in pairs}

    def _split_extensions_configs(
        self,
        extensions: Iterable[str],
    ) -> Iterator[_name_and_config]:
        """Splits the configuration options from a list of strings.

        :returns: a generator of (name, config) tuples
        """
        for extension in extensions:
            yield self._split_extension_config(extension)

    def _apply_extensions(
        self,
        document_extensions: Iterable[_name_and_config] | None = None,
    ) -> None:
        extensions = self.global_extensions.copy()
        extensions.extend(self._split_extensions_configs(self.requested_extensions))
        if document_extensions is not None:
            extensions.extend(document_extensions)

        extension_names = {"markdown.extensions.extra", "mdx_math"}
        extension_configs = {}

        for name, config in extensions:
            if name == "mathjax":
                mathjax_config = {"enable_dollar_delimiter": True}
                extension_configs["mdx_math"] = mathjax_config
            elif name == "remove_extra":
                if "markdown.extensions.extra" in extension_names:
                    extension_names.remove("markdown.extensions.extra")
                if "mdx_math" in extension_names:
                    extension_names.remove("mdx_math")
            else:
                if name in _canonicalized_ext_names:
                    canonical_name = _canonicalized_ext_names[name]
                else:
                    candidate = self._canonicalize_extension_name(name)
                    if candidate is None:
                        warnings.warn(
                            f'Extension "{name}" does not exist.',
                            ImportWarning,
                        )
                        continue
                    canonical_name = candidate
                    _canonicalized_ext_names[name] = canonical_name
                extension_names.add(canonical_name)
                extension_configs[canonical_name] = config
        self.md = self.markdown.Markdown(
            extensions=sorted(extension_names),
            extension_configs=extension_configs,
            output_format="html5",
        )
        self.extensions = extension_names
        self.extension_configs = extension_configs

    def __init__(
        self,
        filename: str | None = None,
        extensions: list[str] | None = None,
    ):
        AbstractMarkup.__init__(self, filename)
        import markdown

        self.markdown = markdown
        self.requested_extensions = extensions or []
        self.global_extensions: list[_name_and_config] = []
        if extensions is None:
            self.global_extensions.extend(self._get_global_extensions(filename))
        self._apply_extensions()

    def convert(self, text: str) -> ConvertedMarkdown:
        # Determine body
        self.md.reset()
        self._apply_extensions(self._get_document_extensions(text))
        body = self.md.convert(text) + "\n"

        # Determine title
        if hasattr(self.md, "Meta") and "title" in self.md.Meta:
            title = str.join(" ", self.md.Meta["title"])
        else:
            title = ""

        # Determine stylesheet
        css_class = None

        if "markdown.extensions.codehilite" in self.extensions:
            config = self.extension_configs.get("markdown.extensions.codehilite", {})
            css_class = config.get("css_class", "codehilite")
            stylesheet = common.get_pygments_stylesheet(f".{css_class}")
        elif "pymdownx.highlight" in self.extensions:
            config = self.extension_configs.get("pymdownx.highlight", {})
            css_class = config.get("css_class", "highlight")
            stylesheet = common.get_pygments_stylesheet(f".{css_class}")
        else:
            stylesheet = ""

        return ConvertedMarkdown(body, title, stylesheet)


class ConvertedMarkdown(ConvertedMarkup):
    def get_javascript(self, webenv: bool = False) -> str:
        if '<script type="math/' not in self.body:
            return ""
        mathjax_url, mathjax_version = common.get_mathjax_url_and_version(webenv)
        config = MATHJAX3_CONFIG if mathjax_version == 3 else MATHJAX2_CONFIG
        async_attr = " async" if mathjax_version == 3 else ""
        script_tag = '<script type="text/javascript" src="%s"%s></script>'
        return config + script_tag % (mathjax_url, async_attr)



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2013-2024

import importlib

import markups.common as common
from markups.abstract import AbstractMarkup, ConvertedMarkup


class TextileMarkup(AbstractMarkup):
    """Markup class for Textile language.
    Inherits :class:`~markups.abstract.AbstractMarkup`.
    """

    name = "Textile"
    attributes = {
        common.LANGUAGE_HOME_PAGE: "https://textile-lang.com",
        common.MODULE_HOME_PAGE: "https://github.com/textile/python-textile",
        common.SYNTAX_DOCUMENTATION: "https://movabletype.org/documentation/author/textile-2-syntax.html",
    }

    file_extensions = (".textile",)
    default_extension = ".textile"

    @staticmethod
    def available() -> bool:
        try:
            importlib.import_module("textile")
        except ImportError:
            return False
        return True

    def __init__(self, filename: str | None = None):
        AbstractMarkup.__init__(self, filename)
        from textile import textile

        self.textile = textile

    def convert(self, text: str) -> ConvertedMarkup:
        return ConvertedMarkup(self.textile(text))



# This file is part of python-markups module
# License: 3-clause BSD, see LICENSE file
# Copyright: (C) Dmitry Shachnev, 2012-2024

from importlib.metadata import entry_points
from typing import Literal, overload

from markups.abstract import AbstractMarkup
from markups.asciidoc import AsciiDocMarkup
from markups.markdown import MarkdownMarkup
from markups.restructuredtext import ReStructuredTextMarkup
from markups.textile import TextileMarkup

__version_tuple__ = (4, 1, 1)
__version__ = ".".join(map(str, __version_tuple__))

__all__ = [
    "AbstractMarkup",
    "AsciiDocMarkup",
    "MarkdownMarkup",
    "ReStructuredTextMarkup",
    "TextileMarkup",
    "find_markup_class_by_name",
    "get_all_markups",
    "get_available_markups",
    "get_markup_for_file_name",
]

builtin_markups = [
    MarkdownMarkup,
    ReStructuredTextMarkup,
    TextileMarkup,
    AsciiDocMarkup,
]

# Public API


def get_all_markups() -> list[type[AbstractMarkup]]:
    """
    :returns: list of all markups (both standard and custom ones)
    """
    entrypoints = entry_points(group="pymarkups")
    return [entry_point.load() for entry_point in entrypoints]


def get_available_markups() -> list[type[AbstractMarkup]]:
    """
    :returns: list of all available markups (markups whose
              :meth:`~markups.abstract.AbstractMarkup.available`
              method returns True)
    """
    available_markups = []
    for markup in get_all_markups():
        if markup.available():
            available_markups.append(markup)
    return available_markups


@overload
def get_markup_for_file_name(
    filename: str,
    return_class: Literal[False] = False,
) -> AbstractMarkup | None: ...


@overload
def get_markup_for_file_name(
    filename: str,
    return_class: Literal[True],
) -> type[AbstractMarkup] | None: ...


def get_markup_for_file_name(
    filename: str,
    return_class: bool = False,
) -> AbstractMarkup | type[AbstractMarkup] | None:
    """
    :param filename: name of the file
    :param return_class: if true, this function will return
                         a class rather than an instance

    :returns: a markup with
              :attr:`~markups.abstract.AbstractMarkup.file_extensions`
              attribute containing extension of `filename`, if found,
              otherwise ``None``

    >>> import markups
    >>> markup = markups.get_markup_for_file_name('foo.mkd')
    >>> markup.convert('**Test**').get_document_body()
    '<p><strong>Test</strong></p>\\n'
    >>> markups.get_markup_for_file_name('bar.rst', return_class=True)
    <class 'markups.restructuredtext.ReStructuredTextMarkup'>
    """
    markup_class = None
    for markup in get_all_markups():
        for extension in markup.file_extensions:
            if filename.endswith(extension):
                markup_class = markup
    if return_class:
        return markup_class
    if markup_class and markup_class.available():
        return markup_class(filename=filename)
    return None


def find_markup_class_by_name(name: str) -> type[AbstractMarkup] | None:
    """
    :returns: a markup with
              :attr:`~markups.abstract.AbstractMarkup.name`
              attribute matching `name`, if found, otherwise ``None``

    >>> import markups
    >>> markups.find_markup_class_by_name('textile')
    <class 'markups.textile.TextileMarkup'>
    """
    for markup in get_all_markups():
        if markup.name.lower() == name.lower():
            return markup
    return None
